
#include <Arduino_AdvancedAnalog.h>

#define BUFSIZE 8 // tampon pour l'ADC
#define NBUF 4
uint32_t sample_rate = 100000;
AdvancedADC adc1(A0);

#define PERIODBUFSIZE 100 // tampon pour les périodes
uint32_t period[2][PERIODBUFSIZE];
uint32_t period_index;
uint8_t period_nbuf;
bool period_buffer_ready = false;

// transformation nombre -> tension
float offset = 2048;
float num2volt = 3.217/4096;

int8_t signe;
bool derive = false;
float epsilon = 1e-3;
uint32_t sample_count;
float last_sample;

#define BSIZE 41
#define ASIZE 1
#define WSIZE 41 // taille maximale de a et b
float b[41] = {-0,-6.05231e-05,-0.00041093,-0.000968941,-0.00111489,1.13996e-18,0.00275013,0.0063475,0.00870225,0.00716282,-3.89207e-18,-0.0120015,-0.0247287,-0.0313913,-0.0247172,6.64417e-18,0.0422416,0.0952615,0.14743,0.185655,0.199687,0.185655,0.14743,0.0952615,0.0422416,6.64417e-18,-0.0247172,-0.0313913,-0.0247287,-0.0120015,-3.89207e-18,0.00716282,0.00870225,0.0063475,0.00275013,1.13996e-18,-0.00111489,-0.000968941,-0.00041093,-6.05231e-05,-0};
float a[1] = {1.0};

float w[WSIZE];
uint16_t iw; // indice de wn

void setup() {
  pinMode(13,OUTPUT);
  Serial.begin(115200);
  while(!Serial);
  for (int i=0; i<WSIZE; i++) w[i] = 0.0f;
  iw = 0;
  last_sample = 0;
  signe = 2;
  sample_count = 0;
  period_index = 0;
  period_nbuf = 0;
  if (!adc1.begin(AN_RESOLUTION_12, sample_rate, BUFSIZE, NBUF)) {
        Serial.println("Impossible d'ouvrir ADC");
        while (1);
    }
}

void analyse() {
  GPIOH->BSRR |= GPIO_BSRR_BS6; //digitalWrite(13,HIGH);
  SampleBuffer buf1 = adc1.read();
  uint16_t *buf1_data = buf1.data();
  int16_t jw;
  float y;
  float ytemp;
  for (int i=0; i<buf1.size(); i++) {
          w[iw] = 1.0/a[0]*(buf1_data[i]-offset)*num2volt;
          jw = iw-1;
          if (jw==-1) jw=WSIZE-1;
          for (uint16_t k=1; k<ASIZE; k++) {
              w[iw] -=  a[k]*w[jw];
              jw = iw-1;
              if (jw==-1) jw=WSIZE-1;
          }
          jw = iw;
          y = 0.0f;
          for (uint16_t k=0; k<BSIZE; k++) {
              y += b[k]*w[jw];
              jw--;
              if (jw==-1) jw=WSIZE-1;
          }
          iw++;
          if (iw==WSIZE) iw=0;
          if (derive) {
            ytemp = y;
            y = (y-last_sample)*sample_rate; // dérivée
            last_sample = ytemp;
          }
          sample_count++;
          if (signe==2) { // initialisation
            if (y>epsilon) signe=1; 
            else if (y<-epsilon) signe=-1;
          }
          else {
            if ((signe>0)&&(y<-epsilon)) signe=-1;
            else if ((signe<0)&&(y>epsilon)) { // passage par zéro par valeur croissante
              signe=1;
              period[period_nbuf][period_index] = sample_count; // sauvegarde dans le tampon
              sample_count = 0;
              period_index++;
              if (period_index==PERIODBUFSIZE) {
                period_index = 0;
                if (period_nbuf==0) period_nbuf=1; else period_nbuf = 0;
                period_buffer_ready = true;
              }
              
            }
          }
        }
    buf1.release();
    GPIOH->BSRR |= GPIO_BSRR_BR6; //digitalWrite(13,LOW);
}

void loop() {
  if (adc1.available()) analyse();
  
  if (period_buffer_ready) {
    period_buffer_ready = false;
    uint8_t nbuf;
    if (period_nbuf==0) nbuf=1; else nbuf=0;
    for (int i=0; i<PERIODBUFSIZE; i++) {
      Serial.print(period[nbuf][i]);
      Serial.print(", ");
    }
  }
  
}

                    